@@ -5,9 +5,9 @@ Module for functions with high level of abstraction
 and/or complexity.
 """
 
+import time
 from fnmatch import fnmatchcase
 # fnmatch.fnmatch would unnecessarily replace path.sep
-# and incorrectly call ntpath.normcase
 
 from . import low
 from . import medium
@@ -16,10 +16,42 @@ os = low.os  # os module
 sys = medium.sys
 
 n = '\n\n'
-if '--verbose' not in sys.argv:
+if '--git-verbose' not in sys.argv:
     # sending to stderr would break tests
-    # and this is a server, its not supposed to have stdout
-    sys.stdout = None
+    # and I have already messed with sys.stdout in server.py
+    def print(*args, **kwargs):
+        pass
+
+
+class DirEntryWrapper:
+    # os.DirEntry is a final class, can't be subclassed
+    __slots__ = 'entry', 'name', 'path'
+
+    def __init__(self, entry: os.DirEntry):
+        self.entry = entry
+        self.name = entry.name.lower()
+        self.path = entry.path.lower()
+
+    def stat(self):
+        try:
+            return self.entry.stat()
+        except PermissionError:
+            return self.entry.stat(follow_symlinks=False)
+
+    def is_file(self):
+        try:
+            return self.entry.is_file()
+        except PermissionError:
+            return self.entry.is_file(follow_symlinks=False)
+
+    def is_dir(self):
+        try:
+            return self.entry.is_dir()
+        except PermissionError:
+            return self.entry.is_dir(follow_symlinks=False)
+
+    def is_symlink(self):
+        return self.entry.is_symlink()
 
 
 class FallbackError(Exception):
@@ -31,12 +63,25 @@ class High(medium.Medium):
     __slots__ = ('git_dir', 'branch', 'fixed', 'ignored', 'packs_list'
                  'untracked', 'staged', 'modified', 'deleted')
 
-    def __init__(self, git_dir: str, branch: str) -> None:
-        self.git_dir = git_dir
+    def __init__(self) -> None:
+        self.untracked = 0
+        self.staged = 0
+        self.modified = 0
+        self.deleted = 0
+
+    def init(self, git_dir, branch):
+        self.git_dir = git_dir.lower()
         self.branch = branch
 
-        print('GIT_DIR:', git_dir)
-        print('Branch', branch, end=n)
+    def status(self):
+        """
+        Do a inline version of 'git status' without any use of git.
+        return: str | None: String with the status, None if there's
+                            nothing to report.
+        """
+
+        print('GIT_DIR:', self.git_dir)
+        print('Branch', self.branch, end=n)
 
         try:
             self.set_index_tracked()
@@ -46,24 +91,30 @@ class High(medium.Medium):
 
         self.set_split_ignored()
         self.set_packs()
-        print('List of packfiles:', self.packs_list, sep='\n', end=n)
+        print('Ignored patterns:')
+        for pat in self.ignored:
+            print(pat)
+        for pat in self.fixed:
+            print(pat)
+        print()
+
+        print('List of packfiles:')
+        for pat in self.packs_list:
+            print(pat)
+        print()
+
+        self.get_full_status(recurse_path=self.git_dir)
+
+        status_string = self.get_status_string(
+            (self.untracked, self.staged, self.modified, self.deleted)
+        )
 
         self.untracked = 0
         self.staged = 0
         self.modified = 0
         self.deleted = 0
 
-    def status(self):
-        """
-        Do a inline version of 'git status' without any use of git.
-        return: str | None: String with the status, None if there's
-                            nothing to report.
-        """
-        self.get_full_status(recurse_path=self.git_dir)
-
-        return self.get_status_string(
-            (self.untracked, self.staged, self.modified, self.deleted)
-        )
+        return status_string
 
     def get_full_status(
         self,
@@ -93,7 +144,8 @@ class High(medium.Medium):
 
         try:
             directory = os.scandir(recurse_path)
-        except PermissionError:
+        # if symlink (a file) points to a dir
+        except (PermissionError, NotADirectoryError):
             return
 
         # index will have only the full path of blobs
@@ -101,54 +153,27 @@ class High(medium.Medium):
         found_in_tree = 0
         for file in directory:
 
-            if file.is_symlink():
-                is_file = True
-                # prevents PermissionError when calling .is_file()
-            else:
-                is_file = file.is_file()
+            file = DirEntryWrapper(file)
 
             relpath = file.path.removeprefix(
                 self.git_dir + '\\'
             ).replace('\\', '/', -1)
 
             # 100755(exe) is file
-            if is_file and relpath in self.index_tracked:
-                # get the item or it's staged
-                for item in tree_items_list:
-                    if item[2] == file.name:
-                        break
-                else:
-                    print('Staged:', file.name, "isn't under a commit.")
-                    self.staged += 1
-                    continue
-
-                found_in_tree += 1
-
-                # syscall mtime == mtime
-                if file.stat().st_mtime == self.index_tracked[relpath]:
-                    continue
-
-                file_hash = self.get_hash_of_file(file.path)
-                file_hash_in_tree = item[1]
-
-                if file_hash_in_tree == file_hash:
-                    continue
-                elif file_hash_in_tree == self.get_hash_of_file(file.path,
-                                                                use_cr=True):
-                    continue
-                self.modified += 1
-
-                print('Modified:', file.name,
-                      f"has different sha1: {file_hash}")
+            if file.is_file() and relpath in self.index_tracked:
+                found_in_tree += self.handle_tracked_file(
+                    file, tree_items_list, relpath
+                )
 
             elif any(fnmatchcase(file.name, pattern)
                      for pattern in self.ignored):
                 pass
-            elif any(fnmatchcase(relpath, pattern)
-                     for pattern in self.fixed):
+            elif any(fnmatchcase(
+                    relpath + '/' if pattern.endswith('/') else relpath,
+                    pattern) for pattern in self.fixed):
                 pass
 
-            elif is_file:  # and relpath not in index
+            elif file.is_file():  # and relpath not in index
                 print('Untracked:', relpath)
                 self.untracked += 1
 
@@ -170,8 +195,8 @@ class High(medium.Medium):
                 # untrack must be setted only if any children is not ignored
                 result = self.handle_untracked_dir(file.path)
                 if result:
-                    self.untracked += result
                     print('Untracked:', relpath)
+                    self.untracked += result
 
             # is_dir and is in tree
             else:
@@ -191,7 +216,43 @@ class High(medium.Medium):
         # endfor
         self.deleted += len(tree_items_list) - found_in_tree
 
+    def handle_tracked_file(self, file, tree_items_list, relpath):
+        local_found_in_tree = 0
+
+        # get the item or it's staged
+        for item in tree_items_list:
+            if item[2] == file.name:
+                break
+        else:
+            print('Staged:', relpath, "isn't under a commit.")
+            self.staged += 1
+            return 0  # or local_found_in_tree
+
+        local_found_in_tree = 1
+
+        mtime = self.index_tracked[relpath]
+        if file.stat().st_mtime != mtime:
+            print('Modified:', relpath)
+            self.modified += 1
+            return local_found_in_tree
+
+        file_hash = self.get_hash_of_file(file.path)
+        file_hash_in_tree = item[1]
+
+        if file_hash_in_tree == file_hash:
+            os.utime(file.path, (time.time(), mtime))
+            return local_found_in_tree
+        elif file_hash_in_tree == self.get_hash_of_file(file.path,
+                                                        use_cr=True):
+            os.utime(file.path, (time.time(), mtime))
+            return local_found_in_tree
+
+        print('Modified:', relpath)
+        self.modified += 1
+        return local_found_in_tree
+
     def handle_untracked_dir(self, dir_path: str) -> int:
+        # although it is untracked, files here can be staged
         """
         Handles when `get_full_status` finds untracked directory.
         param `dir_name`: The path of the untracked directory.
@@ -200,14 +261,29 @@ class High(medium.Medium):
         """
         try:
             directory = os.scandir(dir_path)
-        except PermissionError:
+        except (PermissionError, NotADirectoryError):
             return 0
 
         sub_dir = []
         for file in directory:
             if file.name == '.git':
                 return 0
-            sub_dir.append(file)
+            sub_dir.append(DirEntryWrapper(file))
+
+        # if sub_file is in tracked, dont flag untracked dir
+        staged_flag = False
+        for sub_file in sub_dir:
+            sub_relpath = sub_file.path.removeprefix(
+                self.git_dir + '\\'
+            ).replace('\\', '/', -1)
+
+            if sub_relpath in self.index_tracked:
+                print('Staged:', sub_relpath)
+                self.staged += 1
+                staged_flag = True
+
+        if staged_flag:
+            return 0
 
         local_untracked = 0
         for sub_file in sub_dir:
@@ -226,8 +302,10 @@ class High(medium.Medium):
                     self.git_dir + '\\'
                 ).replace('\\', '/', -1)
 
-                if any(fnmatchcase(sub_relpath, pattern)
-                       for pattern in self.fixed):
+                if any(fnmatchcase(
+                        sub_relpath + '/'
+                        if pattern.endswith('/') else sub_relpath,
+                        pattern) for pattern in self.fixed):
                     continue
                 return 1
 
